@@ -64,7 +64,7 @@
 #include "gtktreemodelfilter.h"
 #include "gtktreeselection.h"
 #include "gtktreestore.h"
-#include "gtktooltips.h"
+#include "gtktooltip.h"
 #include "gtktypebuiltins.h"
 #include "gtkvbox.h"
 
@@ -99,6 +99,8 @@ struct _GtkRecentChooserDefault
   guint show_tips : 1;
   guint show_icons : 1;
   guint local_only : 1;
+
+  guint limit_set : 1;
   
   GSList *filters;
   GtkRecentFilter *current_filter;
@@ -109,8 +111,6 @@ struct _GtkRecentChooserDefault
   gpointer sort_data;
   GDestroyNotify sort_data_destroy;
 
-  GtkTooltips *tooltips;
-
   GtkIconTheme *icon_theme;
   
   GtkWidget *recent_view;
@@ -223,6 +223,7 @@ static void set_current_filter        (GtkRecentChooserDefault *impl,
 static GtkIconTheme *get_icon_theme_for_widget (GtkWidget   *widget);
 static gint          get_icon_size_for_widget  (GtkWidget   *widget,
 						GtkIconSize  icon_size);
+static gint          get_recent_files_limit    (GtkWidget   *widget);
 
 static void reload_recent_items (GtkRecentChooserDefault *impl);
 static void chooser_set_model   (GtkRecentChooserDefault *impl);
@@ -277,6 +278,14 @@ static void     recent_view_drag_data_get_cb      (GtkWidget        *widget,
 						   guint             info,
 						   guint32           time_,
 						   gpointer          data);
+static gboolean recent_view_query_tooltip_cb      (GtkWidget        *widget,
+                                                   gint              x,
+                                                   gint              y,
+                                                   gboolean          keyboard_tip,
+                                                   GtkTooltip       *tooltip,
+                                                   gpointer          user_data);
+
+
 
 G_DEFINE_TYPE_WITH_CODE (GtkRecentChooserDefault,
 			 _gtk_recent_chooser_default,
@@ -345,9 +354,6 @@ _gtk_recent_chooser_default_init (GtkRecentChooserDefault *impl)
   
   impl->current_filter = NULL;
 
-  impl->tooltips = gtk_tooltips_new ();
-  g_object_ref_sink (impl->tooltips);
-  
   impl->recent_items = NULL;
   impl->n_recent_items = 0;
   impl->loaded_items = 0;
@@ -373,6 +379,8 @@ gtk_recent_chooser_default_constructor (GType                  type,
   g_assert (impl->manager);
   
   gtk_widget_push_composite_child ();
+
+  impl->limit = get_recent_files_limit (GTK_WIDGET (impl));
   
   scrollw = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrollw),
@@ -395,6 +403,10 @@ gtk_recent_chooser_default_constructor (GType                  type,
   g_signal_connect (impl->recent_view, "drag_data_get",
 		    G_CALLBACK (recent_view_drag_data_get_cb), impl);
 
+  g_object_set (impl->recent_view, "has-tooltip", TRUE, NULL);
+  g_signal_connect (impl->recent_view, "query-tooltip",
+                    G_CALLBACK (recent_view_query_tooltip_cb), impl);
+
   g_object_set_data (G_OBJECT (impl->recent_view),
                      "GtkRecentChooserDefault", impl);
   
@@ -449,10 +461,8 @@ gtk_recent_chooser_default_constructor (GType                  type,
   gtk_combo_box_set_focus_on_click (GTK_COMBO_BOX (impl->filter_combo), FALSE);
   g_signal_connect (impl->filter_combo, "changed",
                     G_CALLBACK (filter_combo_changed_cb), impl);
-  gtk_tooltips_set_tip (impl->tooltips,
-			impl->filter_combo,
-		        _("Select which type of documents are shown"),
-			NULL);
+  gtk_widget_set_tooltip_text (impl->filter_combo,
+		               _("Select which type of documents are shown"));
   
   gtk_box_pack_end (GTK_BOX (impl->filter_combo_hbox),
                     impl->filter_combo,
@@ -501,11 +511,6 @@ gtk_recent_chooser_default_set_property (GObject      *object,
       break;
     case GTK_RECENT_CHOOSER_PROP_SHOW_TIPS:
       impl->show_tips = g_value_get_boolean (value);
-
-      if (impl->show_tips)
-        gtk_tooltips_enable (impl->tooltips);
-      else
-        gtk_tooltips_disable (impl->tooltips);
       break;
     case GTK_RECENT_CHOOSER_PROP_SHOW_ICONS:
       impl->show_icons = g_value_get_boolean (value);
@@ -525,6 +530,7 @@ gtk_recent_chooser_default_set_property (GObject      *object,
       break;
     case GTK_RECENT_CHOOSER_PROP_LIMIT:
       impl->limit = g_value_get_int (value);
+      impl->limit_set = TRUE;
       reload_recent_items (impl);
       break;
     case GTK_RECENT_CHOOSER_PROP_SORT_TYPE:
@@ -626,12 +632,6 @@ gtk_recent_chooser_default_dispose (GObject *object)
       impl->recent_store = NULL;
     }
 
-  if (impl->tooltips)
-    {
-      g_object_unref (impl->tooltips);
-      impl->tooltips = NULL;
-    }
-
   G_OBJECT_CLASS (_gtk_recent_chooser_default_parent_class)->dispose (object);
 }
 
@@ -864,19 +864,26 @@ cleanup_after_load (gpointer user_data)
 static void
 reload_recent_items (GtkRecentChooserDefault *impl)
 {
+  GtkWidget *widget;
+
   /* reload is already in progress - do not disturb */
   if (impl->load_id)
     return;
   
+  widget = GTK_WIDGET (impl);
+
   gtk_tree_view_set_model (GTK_TREE_VIEW (impl->recent_view), NULL);
   gtk_list_store_clear (impl->recent_store);
   
   if (!impl->icon_theme)
-    impl->icon_theme = get_icon_theme_for_widget (GTK_WIDGET (impl));
+    impl->icon_theme = get_icon_theme_for_widget (widget);
 
-  impl->icon_size = get_icon_size_for_widget (GTK_WIDGET (impl),
+  impl->icon_size = get_icon_size_for_widget (widget,
 		  			      GTK_ICON_SIZE_BUTTON);
 
+  if (!impl->limit_set)
+    impl->limit = get_recent_files_limit (widget);
+
   set_busy_cursor (impl, TRUE);
 
   impl->load_state = LOAD_EMPTY;
@@ -963,6 +970,8 @@ recent_icon_data_func (GtkTreeViewColumn *tree_column,
   
   if (pixbuf)  
     g_object_unref (pixbuf);
+
+  gtk_recent_info_unref (info);
 }
 
 static void
@@ -973,8 +982,7 @@ recent_meta_data_func (GtkTreeViewColumn *tree_column,
 		       gpointer           user_data)
 {
   GtkRecentInfo *info = NULL;
-  gchar *uri, *name;
-  gchar *str;
+  gchar *name;
   
   gtk_tree_model_get (model, iter,
                       RECENT_DISPLAY_NAME_COLUMN, &name,
@@ -982,18 +990,11 @@ recent_meta_data_func (GtkTreeViewColumn *tree_column,
                       -1);
   g_assert (info != NULL);
   
-  uri = gtk_recent_info_get_uri_display (info);
-  
   if (!name)
     name = gtk_recent_info_get_short_name (info);
 
-  str = g_strconcat ("<b>", name, "</b>\n",
-                     "<small>", _("Location:"), " ", uri, "</small>",
-                     NULL);
+  g_object_set (cell, "text", name, NULL);
   
-  g_object_set (cell, "markup", str, NULL);
-  
-  g_free (uri);
   g_free (name);
   gtk_recent_info_unref (info);
 }
@@ -1267,7 +1268,7 @@ gtk_recent_chooser_default_add_filter (GtkRecentChooser *chooser,
   /* display new filter */
   name = gtk_recent_filter_get_name (filter);
   if (!name)
-    name = "Untitled filter";
+    name = _("Untitled filter");
     
   gtk_combo_box_append_text (GTK_COMBO_BOX (impl->filter_combo), name);
   
@@ -1397,6 +1398,21 @@ get_icon_size_for_widget (GtkWidget   *widget,
   return FALLBACK_ICON_SIZE;
 }
 
+static gint
+get_recent_files_limit (GtkWidget *widget)
+{
+  GtkSettings *settings;
+  gint limit;
+
+  if (gtk_widget_has_screen (widget))
+    settings = gtk_settings_get_for_screen (gtk_widget_get_screen (widget));
+  else
+    settings = gtk_settings_get_default ();
+  
+  g_object_get (G_OBJECT (settings), "gtk-recent-files-limit", &limit, NULL);
+
+  return limit;
+}
 
 static void
 recent_manager_changed_cb (GtkRecentManager *manager,
@@ -1540,7 +1556,54 @@ recent_view_drag_data_get_cb (GtkWidget        *widget,
   g_free (drag_data);
 }
 
+static gboolean
+recent_view_query_tooltip_cb (GtkWidget  *widget,
+                              gint        x,
+                              gint        y,
+                              gboolean    keyboard_tip,
+                              GtkTooltip *tooltip,
+                              gpointer    user_data)
+{
+  GtkRecentChooserDefault *impl = user_data;
+  GtkTreeView *tree_view;
+  GtkTreeIter iter;
+  GtkTreePath *path = NULL;
+  GtkRecentInfo *info = NULL;
+  gchar *uri_display;
 
+  if (!impl->show_tips)
+    return FALSE;
+
+  tree_view = GTK_TREE_VIEW (impl->recent_view);
+
+  gtk_tree_view_get_tooltip_context (tree_view,
+                                     &x, &y,
+                                     keyboard_tip,
+                                     NULL, &path, NULL);
+  if (!path)
+    return FALSE;
+
+  if (!gtk_tree_model_get_iter (GTK_TREE_MODEL (impl->recent_store), &iter, path))
+    {
+      gtk_tree_path_free (path);
+      return FALSE;
+    }
+
+  gtk_tree_model_get (GTK_TREE_MODEL (impl->recent_store), &iter,
+                      RECENT_INFO_COLUMN, &info,
+                      -1);
+
+  uri_display = gtk_recent_info_get_uri_display (info);
+  
+  gtk_tooltip_set_text (tooltip, uri_display);
+  gtk_tree_view_set_tooltip_row (tree_view, tooltip, path);
+
+  g_free (uri_display);
+  gtk_tree_path_free (path);
+  gtk_recent_info_unref (info);
+
+  return TRUE;
+}
 
 static void
 remove_selected_from_list (GtkRecentChooserDefault *impl)
